import json

from dashpaan.elements.base import Element


class Tabular(Element):
    kind = "tabular"

    tabs = []

    def json(self):
        return {
            **super(Tabular, self).json(),
            "version": self.version,
            "tabs": self.tabs
        }

    @classmethod
    def from_json(cls, obj):
        return Tabular(**obj)



import json

from dashpaan.elements.base import Element


class TextAreaEditor(Element):
    kind = "text-area-editor"

    name = ""

    def json(self):
        return {
            **super(TextAreaEditor, self).json(),
            "version": self.version,
            "name": self.name
        }

    @classmethod
    def from_json(cls, obj):
        return TextAreaEditor(**obj)



import json

from dashpaan.elements.base import Element


class Button(Element):
    kind = "button"

    title = ""
    color = ""
    data = True
    action = {}

    def json(self):
        return {
            **super(Button, self).json(),
            "title": self.title,
            "color": self.color,
            "action": self.action,
            "data": self.data
        }

    @classmethod
    def from_json(cls, obj):
        return Button(**obj)


from dashpaan.elements.heading import Heading


def convert(obj):
    if "kind" in obj:
        if obj["kind"] == "heading":
            return Heading.from_json(obj)

    raise ValueError("Not Valid Element")



from json import *
import json as origin

from datetime import datetime

from ..elements.base import Element
from dashpaan.elements.reverse import convert


class DashpaanJSONEncoder(origin.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, Element):
            return obj.json()

        if isinstance(obj, datetime):
            # Handle datetime serialization
            return obj.isoformat()

        # Fallback to the default serialization
        return super().default(obj)


class DashpaanJSONDecoder(origin.JSONDecoder):
    def __init__(self, *args, **kwargs):
        super().__init__(object_hook=self.object_hook, *args, **kwargs)

    def object_hook(self, obj):
        # Deserialize Element
        if "kind" in obj and "version" in obj:
            return convert(obj)

        # Deserialize datetime (assuming ISO 8601 format)
        for key, value in obj.items():
            if isinstance(value, str):
                try:
                    obj[key] = datetime.fromisoformat(value)
                except ValueError:
                    pass

        return obj


JSONEncoder = DashpaanJSONEncoder
JSONDecoder = DashpaanJSONDecoder


def dumps(obj, *args, **kwargs):
    return origin.dumps(obj, cls=DashpaanJSONEncoder, *args, **kwargs)


def loads(obj, *args, **kwargs):
    return origin.loads(obj, cls=DashpaanJSONDecoder, *args, **kwargs)


__all__ = [
    'dump', 'dumps', 'load', 'loads',
    'JSONDecoder', 'JSONDecodeError', 'JSONEncoder',
]



from dashpaan.elements.base import Element


class Card(Element):
    kind = "card"

    icons = []
    cover = ""
    size = ""
    title = ""
    boxes = []
    menu = {}
    description = ""

    def json(self):
        return {
            **super(Card, self).json(),
            "title": self.title,
            "size": self.size,
            "icons": self.icons,
            "boxes": self.boxes,
            "menu": self.menu,
            "description": self.description,
            "cover": self.cover
        }

    @classmethod
    def from_json(cls, obj):
        return Card(**obj)



import json

from dashpaan.elements.base import Element


class Navigation(Element):
    kind = "navigation"

    items = []

    def json(self):
        return {
            **super(Navigation, self).json(),
            "items": self.items
        }

    @classmethod
    def from_json(cls, obj):
        return Navigation(**obj)



import json

from dashpaan.elements.base import Element


class FormElementUploadImage(Element):
    kind = "formElementUploadImage"

    uri = ""
    title = ""
    data = [
        {
            "id": "01ff61ca-2e47-4ebe-b076-7aa9491fcbcb",
            "payload": {},
            "headers": {},
            "url": ""
        }
    ]
    variables = {
        "bookName": "The Hard Things about Hard Things",
        "categoryId": 87564521
    }

    def json(self):
        return {
            **super(FormElementUploadImage, self).json(),
            "uri": self.uri,
            "title": self.title,
            "data": self.data,
            "variables": self.variables
        }

    @classmethod
    def from_json(cls, obj):
        return FormElementUploadImage(**obj)



import json

from dashpaan.elements.base import Element


class DataTable(Element):
    kind = "data-table"

    uri = ""
    title = ""
    errors = {}
    columns = []
    rows = []

    def json(self):
        return {
            **super(DataTable, self).json(),
            "uri": self.uri,
            "title": self.title,
            "errors": self.errors,
            "columns": self.columns,
            "rows": self.rows
        }

    @classmethod
    def from_json(cls, obj):
        return DataTable(**obj)


from dashpaan.elements.base import Element


class Information(Element):
    kind = "information"

    title = ""
    size = ""
    information = []

    def json(self):
        return {
            **super(Information, self).json(),
            "title": self.title,
            "size": self.size,
            "information": self.information
        }

    @classmethod
    def from_json(cls, obj):
        return Information(**obj)



import json

from dashpaan.elements.base import Element


class TextField(Element):
    kind = "text-field"

    name = ""
    label = ""
    required = False
    defaultValue = ""

    def json(self):
        return {
            **super(TextField, self).json(),
            "name": self.name,
            "label": self.label,
            "required": self.required,
            "defaultValue": self.defaultValue
        }

    @classmethod
    def from_json(cls, obj):
        return TextField(**obj)


from dashpaan.elements.base import Element


class Select(Element):
    kind = "select"

    name = ""
    default = ""
    multiple = True
    label = ""
    options = []

    def json(self):
        return {
            **super(Select, self).json(),
            "name": self.name,
            "default": self.default,
            "label": self.label,
            "options": self.options
        }

    @classmethod
    def from_json(cls, obj):
        return Select(**obj)



class Element:
    kind = None
    version = "v1"

    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            if hasattr(self, key):
                setattr(self, key, value)

    def json(self):
        return {
            "kind": self.kind,
            "version": self.version
        }



import json

from dashpaan.elements.base import Element


class CircularDistribution(Element):
    kind = "circular-distribution"

    labels = {}
    series = {}

    def json(self):
        return {
            **super(CircularDistribution, self).json(),
            "labels": self.labels,
            "series": self.series
        }

    @classmethod
    def from_json(cls, obj):
        return CircularDistribution(**obj)


import json

from dashpaan.elements.base import Element


class Form(Element):
    kind = "form"
    size = ""
    fields = []
    buttons = []

    def json(self):
        return {
            **super(Form, self).json(),
            "size": self.size,
            "fields": [field.json() for field in self.fields],
            "buttons": [button.json() for button in self.buttons]
        }

    @classmethod
    def from_json(cls, obj):
        return Form(**obj)



import json

from dashpaan.elements.base import Element


class Page(Element):
    kind = "page"

    uri = ""
    title = ""
    data = []
    variables = {}
    templates = {}
    elements = []
    navigation = "inherit"

    def json(self):
        return {
            **super(Page, self).json(),
            "uri": self.uri,
            "title": self.title,
            "data": self.data,
            "variables": self.variables,
            "templates": self.templates,
            "elements": [element.json() for element in self.elements],
            "navigation": self.navigation.json() if type(self.navigation) is list else self.navigation
        }

    @classmethod
    def from_json(cls, obj):
        return Page(**obj)



import json

from dashpaan.elements.base import Element


class QrCode(Element):
    kind = "qr_code"

    size = ""
    url = ""

    def json(self):
        return {
            **super(QrCode, self).json(),
            "url": self.url,
            "size": self.size,

        }

    @classmethod
    def from_json(cls, obj):
        return QrCode(**obj)



import json

from dashpaan.elements.base import Element


class WYSIWYGEditor(Element):
    kind = "wysiwyg-editor"

    name = ""
    content = ""

    def json(self):
        return {
            **super(WYSIWYGEditor, self).json(),
            "name": self.name,
            "content": self.content
        }

    @classmethod
    def from_json(cls, obj):
        return WYSIWYGEditor(**obj)



import json

from dashpaan.elements.base import Element


class PieChart(Element):
    kind = "pie_chart"

    title = ""
    series = {}
    labels = []
    options = {}

    def json(self):
        return {
            **super(PieChart, self).json(),
            "title": self.title,
            "series": self.series,
            "labels": self.labels,
            "options": self.options
        }

    @classmethod
    def from_json(cls, obj):
        return PieChart(**obj)



from dashpaan.elements.base import Element


class Flex(Element):
    kind = "flex"

    class_name = ""
    mode = ""
    breakable = ""
    size = ""
    elements = []

    def json(self):
        return {
            **super(Flex, self).json(),
            "mode": self.mode,
            "breakable": self.breakable,
            "className": self.class_name,
            "size": self.size,
            "elements": [element.json() for element in self.elements]
        }

    @classmethod
    def from_json(cls, obj):
        return Flex(**obj)



import json

from dashpaan.elements.base import Element


class LineChart(Element):
    kind = "chart-line"

    data = {}

    def json(self):
        return {
            **super(LineChart, self).json(),
            "version": self.version,
            "data": self.data
        }

    @classmethod
    def from_json(cls, obj):
        return LineChart(**obj)



from dashpaan.elements.base import Element


class Heading(Element):
    kind = "heading"

    title = ""
    subtitle = ""

    def json(self):
        return {
            **super(Heading, self).json(),
            "title": self.title,
            "subtitle": self.subtitle
        }

    @classmethod
    def from_json(cls, obj):
        return Heading(**obj)



from setuptools import setup, find_packages

setup(
    name='dashpaan',
    version='1.0.1',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        line.strip() for line in open('requirements.txt').readlines() if line.strip()
    ],
    author='Monir.co',
    author_email='connect@monirs.com',
    description='Dashpaan is a Python package for building interactive dashboards using Flask and Django.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/yourusername/dashpaan',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
)



import json

from dashpaan.elements.base import Element


class ChartComparison(Element):
    kind = "wave-chart"

    data = {}

    def json(self):
        return {
            **super(ChartComparison, self).json(),
            "data": self.data
        }

    @classmethod
    def from_json(cls, obj):
        return ChartComparison(**obj)


import json

from dashpaan.elements.base import Element


class WaveChart(Element):
    kind = "wave-chart"

    size = ""
    title = ""
    subtitle = ""
    series = {}

    def json(self):
        return {
            **super(WaveChart, self).json(),
            "size": self.size,
            "title": self.title,
            "subtitle": self.subtitle,
            "series": self.series
        }

    @classmethod
    def from_json(cls, obj):
        return WaveChart(**obj)


import json

from dashpaan.elements.base import Element


class StickyStat(Element):
    kind = "sticky-stat"

    size = ""
    value = 0
    color = ""
    title = ""

    def json(self):
        return {
            **super(StickyStat, self).json(),
            "version": self.version,
            "size": self.size,
            "value": self.value,
            "color": self.color,
            "title": self.title
        }

    @classmethod
    def from_json(cls, obj):
        return StickyStat(**obj)



from dashpaan.elements.base import Element


class StickyTarget(Element):
    kind = "sticky-target"

    size = ""
    color = ""
    data = {}
    more = {}

    def json(self):
        return {
            **super(StickyTarget, self).json(),
            "version": self.version,
            "size": self.size,
            "color": self.color,
            "data": self.data,
            "more": self.more
        }

    @classmethod
    def from_json(cls, obj):
        return StickyTarget(**obj)



import json

from dashpaan.elements.base import Element


class Switch(Element):
    kind = "switch"

    type = ""
    name = ""
    label = ""

    def json(self):
        return {
            **super(Switch, self).json(),
            "type": self.type,
            "name": self.name,
            "label": self.label
        }

    @classmethod
    def from_json(cls, obj):
        return Switch(**obj)